import { exec as cmd } from 'child_process'

/**
 *
 * @param {string} command - The command to execute
 * @param {(out:string, err:string)=>string} callback - The callback to execute on the output
 * @returns {void}
 * @description Execute the command and receive one callback for personalize the output
 */
export const exec = (command, callback) => {
  cmd(command, { cwd: process.cwd() }, (err, stdout, stderr) => {
    if (!callback) callback = defaultCallback
    if (err) return callback(err.message, null)
    if (callback) return callback(stderr, stdout)
    return callback(null, stdout)
  })
}

function defaultCallback(err, out) {
  if (err) return console.error(_.red(err).trim())
  if (out) return console.log(_.green(out.trim()))
}
export const _ = {
  red: text => `\x1b[91m${text}\x1b[0m`,
  green: text => `\x1b[92m${text}\x1b[0m`,
  yellow: text => `\x1b[33m${text}\x1b[0m`,
  blue: text => `\x1b[34m${text}\x1b[0m`,
  cyan: text => `\x1b[36m${text}\x1b[0m`,
  gray: text => `\x1b[90m${text}\x1b[0m`,
}
const { red, green, gray, cyan } = _
export const helpers = {
  commit:
    ` ${cyan`[Description]:`}\n` +
    `   Commit changes to the repository\n\n` +
    ` ${cyan`[Usage]:`}\n` +
    `   $ gx commit [options] [description]\n\n` +
    ` ${cyan`[Options]:`}\n` +
    `     -c: 🔧 chore (generic change)  [default]\n` +
    `     -f: ✨ feat (new feature)\n` +
    `     -b: 🐛 bug (fixing a bug)\n` +
    `     -p: ⚡ perf (improves performance)\n` +
    `     -s: 🎨 style (code style)\n` +
    `     -i: 🚀 rocket (initialize project)\n` +
    `     -r: ♻️ refactor (code improvement)\n` +
    `     -t: 🧪 test (adding tests)\n` +
    `     -d: 📝 docs (changes in documentation)\n` +
    `     -ty: 🏷️ typing (types and interfaces)\n\n` +
    ` ${cyan`[Examples]:`}\n` +
    `   $ gxc ${cyan`-f`}:${red`auth`} ${green`"add jwt auth"`}\n` +
    `   > ${gray`git commit -m ":sparkless: feat(auth): add new auth module"`}\n` +
    `   $ gxc ${cyan`-b`} ${green`"fix bug #1"`}\n` +
    `   > ${gray`git commit -m ":bug: bug: fix bug #1"`}\n`,
  add:
    ` ${cyan`[Description]:`}\n` +
    `   Add files/folders to the repository\n\n` +
    ` ${cyan`[Usage]:`}\n` +
    `   $ gxa [files]\n\n` +
    ` ${cyan`[Examples]:`}\n` +
    `   $ gxa ${green`src/index.js`}\n` +
    `   > ${gray`git add src/index.js`}\n` +
    `   $ gxa ${green`package.json yarn.lock`}\n` +
    `   > ${gray`git add package.json yarn.lock`}\n`,
  push:
    ` ${cyan`[Description]:`}\n` +
    `   Push changes to the remote repository\n\n` +
    ` ${cyan`[Usage]:`}\n` +
    `   $ gxp [options]\n\n` +
    ` ${cyan`[Options]:`}\n` +
    `     -f: 🔧 force push (force push to the remote repository)\n` +
    `     -u: 🔧 update (update the remote repository)\n` +
    `     -h: 🔧 help (show this help)\n\n` +
    ` ${cyan`[Examples]:`}\n` +
    `   $ gxp ${cyan`-f`}\n` +
    `   > ${gray`git push -f`}\n`,
}
