#!/usr/bin/env node
import { exec, _, helpers } from '../core.js'

/**
 * @param {string|undefined} flag
 * @returns {string}
 */
const change = exist => (exist ? `(${exist})` : '')
const commitTypes = {
  '-f': where => `:sparkles: feat${change(where)}: `,
  '-b': where => `:bug: bug${change(where)}: `,
  '-s': where => `:art: style${change(where)}: `,
  '-r': where => `:recycle: refactor${change(where)}: `,
  '-t': where => `:test_tube: test${change(where)}: `,
  '-d': where => `:pencil: docs${change(where)}: `,
  '-i': where => `:tada: improvement${change(where)}: `,
  '-p': where => `:zap: perf${change(where)}: `,
  '-c': where => `:wrench: chore${change(where)}: `,
  '-ty': where => `:label: typing${change(where)}: `,
}
/**
 * @param {string[]} ctx - The command line arguments (Process.argv[2])
 */
function handler(ctx) {
  const input = ctx.join(' ')
  if (input.includes('--help')) {
    return console.log(helpers.commit)
  }
  const typeRegex = /(?<type>-\w):?(?<change>[^\s]+)?/
  const match = input.match(typeRegex)
  const desc = input.replace(typeRegex, '').trim()
  let type, change
  if (!match) (type = '-c'), (change = undefined)
  else {
    type = match.groups.type || '-c'
    change = match.groups.change || undefined
  }
  const message = commitTypes[type](change) + desc
  const command = `git commit -m "${message}"`

  return exec(command)
}
handler(process.argv.slice(2))
